"""Translation constants."""
import pathlib

CORE_PROJECT_ID = "130246255a974bd3b5e8a1.51616605"
FRONTEND_PROJECT_ID = "3420425759f6d6d241f598.13594006"
CLI_2_DOCKER_IMAGE = "v2.3.0"
INTEGRATIONS_DIR = pathlib.Path("homeassistant/components")
FRONTEND_DIR = pathlib.Path("../frontend")
