"""Constants for the devolo_home_control integration."""

DOMAIN = "devolo_home_control"
DEFAULT_MYDEVOLO = "https://www.mydevolo.com"
DEFAULT_MPRM = "https://homecontrol.mydevolo.com"
PLATFORMS = ["switch"]
CONF_MYDEVOLO = "mydevolo_url"
CONF_HOMECONTROL = "home_control_url"
